package NOLookup;

use 5.008001;
use strict;
use warnings;

our $VERSION = "1.16";


=pod 

=encoding ISO-8859-1

=head1 NAME

NOLookup - a set of lookup modules for various Norwegian data services.

=head1 SYNOPSIS

See the POD for the respective lookup modules.

=head1 DESCRIPTION

Search and lookup information about Noriwegian domains, organizations and trademarks.

See POD for each lookup module for further details.

=head1 LICENSE

Copyright (C) 2017- Trond Haugen.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

Trond Haugen E<lt>trondh@uninett.noE<gt>

=cut

1;
